searchData={"items":[{"type":"behaviour","doc":"The Elixir Scribe Typed Contract guarantees the shape of your data throughout your codebase.\n\nUse it as a contract for your data shape to eliminate potential bugs that may be caused by creating the data with the wrong type. This leads to more robust code and fewer tests needed to guarantee data correctness, compared to when a struct or a plain map was previously used.","title":"ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html"},{"type":"behaviour","doc":"Let's imagine that the Business requested a new feature, a Marketing funnel where they only want to allow personas who have a company email to enter the funnel, and they require them to provide a name, email and optionally their role in the company.\n\nWe can use a Typed Contract to translate these business rules into a contract that guarantees data correctness across all our code base.\n\nFor example:\n\n```\ndefmodule Persona do\n  @moduledoc false\n\n  require PersonaValidator\n\n  @fields %{\n    required: [:name, :email],\n    optional: [role: nil]\n  }\n\n  use ElixirScribe.Behaviour.TypedContract, fields: @fields\n\n  @impl true\n  def type_spec() do\n    schema(%__MODULE__{\n      name: is_binary() |> spec(),\n      email: PersonaValidator.corporate_email?() |> spec(),\n      role: PersonaValidator.role?() |> spec()\n    })\n  end\nend\n\n```\n>","title":"Typed Contract - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-typed-contract"},{"type":"behaviour","doc":"> - When defining the `type_spec/0` the schema can use the built-in specs from [Norm](https://hexdocs.pm/norm/Norm.html#module-validation-and-conforming-values) or your own ones.\n> - All specs defined with `PersonaValidator.*` are custom ones.\n\n\n>","title":"Specs {: .info} - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-specs-info"},{"type":"behaviour","doc":"> - A typed contract **MUST** declare a type spec for the required and optional fields.\n> - The optional fields **MUST** have a default value.\n\n>","title":"Required and Optional Fields {: .warning} - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-required-and-optional-fields-warning"},{"type":"behaviour","doc":"> - To take advantage of the safety guarantees offered by the Elixir Scribe Typed Contract, you **MUST** not create or update it directly, as allowed by Elixir. Instead, you need to use the built-in functions `new/1`, `new!/1`, `update/3`, or `update!/3` that will guarantee it conforms with the type specs when one is created or updated.\n> - Use the `conforms?/1` function at the place you use the typed contract to ensure that you still have a struct that conforms with the type spec, because it may have been manipulated directly between the point it was created and where you are using it.\n> - For introspection, the `fields/0` and `type_spec/0` functions are provided.","title":"Contract Guarantees {: .error} - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-contract-guarantees-error"},{"type":"behaviour","doc":"To run the usage examples:\n\n```\niex -S mix\n```","title":"Usage Examples - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-usage-examples"},{"type":"behaviour","doc":"To create a new Persona:\n\n```\niex> Persona.new! %{name: \"Paulo\", email: \"exadra37@company.com\"}\n%Persona{name: \"Paulo\", email: \"exadra37@company.com\", role: nil, self: Persona}\n```\n\nTo update the Persona:\n\n```\niex> persona = Persona.new! %{name: \"Paulo\", email: \"exadra37@company.com\"}\n%Persona{name: \"Paulo\", email: \"exadra37@company.com\", role: nil, self: Persona}\niex> persona.self.update! persona, :role, \"Elixir Scribe Engineer\"\n%Persona{\n  name: \"Paulo\",\n  email: \"exadra37@company.com\",\n  role: \"Elixir Scribe Engineer\",\n  self: Persona\n}\n```\n\nAlternatively, you can use `new/1` and `update/3` which will return a `{:ok, result}` or `{:error, reason}` tuple.","title":"With Valid Data Types - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-with-valid-data-types"},{"type":"behaviour","doc":"Passing the role as an atom, instead of the expected string:\n\n```\niex> Persona.new %{name: \"Paulo\", email: \"exadra37@company.com\", role: :cto}\n{:error, [%{input: :cto, path: [:role], spec: \"PersonaValidator.role?()\"}]}\n```\n\nThe same as above, but using `new!/1` that will raise:\n\n```\niex> Persona.new! %{name: \"Paulo\", email: \"exadra37@company.com\", role: :cto}\n** (Norm.MismatchError) Could not conform input:\nval: :cto in: :role fails: PersonaValidator.role?()\n```\n\nInvoking `update/3` and `update!/3` will output similar results.\n\n>","title":"With Invalid Data Types - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-with-invalid-data-types"},{"type":"behaviour","doc":"> - The Elixir Scribe typed contract acts as a contract for the businesse rules to guarantee data correctness at anypoint it's used in the code base.\n> - Now the developer only needs to test that a Persona complies with this business rules in the test for this contract, because everywhere the Persona contract is used it's guaranteed that the data is in the expected shape.\n> - This translates to fewer bugs, less technical debt creeping in and a more robust code base.","title":"Less Tests and Fewer Bugs {: .tip} - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-less-tests-and-fewer-bugs-tip"},{"type":"behaviour","doc":"When using an Elixir Typed Contract through your code base you may want to ensure that the contract still conforms with the type spec, because it may have been directly manipulated, which may break it's guarantees. This is a limitation of the Elixir language, not of the typed contract implementation.\n\nLet's create a valid Persona typed contract and then check if it still conforms:\n\n```\niex> persona = Persona.new! %{name: \"Paulo\", email: \"exadra37@company.com\"}\n%Persona{name: \"Paulo\", email: \"exadra37@company.com\", role: nil, self: Persona}\niex> # Some layers deeper on your code you can ensure it still conforms:\niex> persona.self.conforms? persona\ntrue\niex> # If you prefer the result as an ok tuple:\niex> persona.self.conforms persona\n{:ok,\n %Persona{\n   name: \"Paulo\",\n   email: \"exadra37@company.com\",\n   role: nil,\n   self: Persona\n }}\niex> # Or if you prefer you can use the bang function:\niex> persona.self.conforms! persona\n%Persona{name: \"Paulo\", email: \"exadra37@company.com\", role: nil, self: Persona}\n```\n\nThis time let's create again a valid Persona typed contract, manipulated it directly with an invalid value and then check if it still conforms:\n\n```\niex> persona = Persona.new! %{name: \"Paulo\", email: \"exadra37@company.com\"}\n%Persona{name: \"Paulo\", email: \"exadra37@company.com\", role: nil, self: Persona}\niex> # Manipulating the struct directly with the invalid value `:dev` for the `:role` field:\niex> persona = %{persona | role: :dev}\n%Persona{\n  name: \"Paulo\",\n  email: \"exadra37@company.com\",\n  role: :dev,\n  self: Persona\n}\niex> # The direct manipulation of a struct allows to update it with invalid values for the contract\niex> # To mitigate the contract violation we just need to check it still conforms:\niex> persona.self.conforms? persona\nfalse\niex> # If you need to know the reason then use instead:\niex> persona.self.conforms persona\n{:error, [%{input: :dev, path: [:role], spec: \"PersonaValidator.role?()\"}]}\niex> # If you prefer to let it crash, then use:\niex> persona.self.conforms! persona\n** (Norm.MismatchError) Could not conform input:\nval: :dev in: :role fails: PersonaValidator.role?()\n```","title":"Validation - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-validation"},{"type":"behaviour","doc":"To introspect the fields used to define the typed contract at compile time:\n\n```\niex> Persona.fields\n%{\n  config: %{\n    extra: [self: Persona],\n    required: [:name, :email],\n    optional: [role: nil]\n  },\n  defstruct: [:name, :email, {:role, nil}, {:self, Persona}]\n}\n```\n\nTo introspect the Norm type spec:\n\n```\niex> Persona.type_spec\n```\n\nThe output:\n\n```\n#Norm.Schema<%Persona{\n  name: #Norm.Spec ,\n  email: #Norm.Spec ,\n  role: #Norm.Spec ,\n  self: Persona\n}>\n```\n\nThe `:self` field is an extra added at compile time by the typed contract macro to allow you to self reference the typed contract like you already noticed in the above examples.","title":"Introspection - ElixirScribe.Behaviour.TypedContract","ref":"ElixirScribe.Behaviour.TypedContract.html#module-introspection"},{"type":"callback","doc":"Accepts the Elixir Scribe Typed Contract itself to check it still conforms with the specs.\n\nReturns `{:ok, struct}` on success, otherwise returns `{:error, reason}`.","title":"ElixirScribe.Behaviour.TypedContract.conforms/1","ref":"ElixirScribe.Behaviour.TypedContract.html#c:conforms/1"},{"type":"callback","doc":"Accepts the Elixir Scribe Typed Contract itself to check it still conforms with the specs.\n\nReturns the typed contract on success, otherwise raises an error.","title":"ElixirScribe.Behaviour.TypedContract.conforms!/1","ref":"ElixirScribe.Behaviour.TypedContract.html#c:conforms!/1"},{"type":"callback","doc":"Accepts the Elixir Scribe Typed Contract itself to check it still conforms with the specs.\n\nCreating or modifying the struct directly can cause it to not be conformant anymore with the specs.\n\nUseful to use by modules operating on the Typed Contract to ensure it wasn't directly modified after being created with `new/1` or `new!/1`.","title":"ElixirScribe.Behaviour.TypedContract.conforms?/1","ref":"ElixirScribe.Behaviour.TypedContract.html#c:conforms?/1"},{"type":"callback","doc":"Returns the fields definition used to define the struct for the Elixir Scribe Typed Contract at compile time.","title":"ElixirScribe.Behaviour.TypedContract.fields/0","ref":"ElixirScribe.Behaviour.TypedContract.html#c:fields/0"},{"type":"callback","doc":"Accepts a map with the required and optional fields to create an Elixir Scribe Typed Contract.\n\nReturns `{:ok, struct}` on successful creation, otherwise returns `{:error, reason}`.","title":"ElixirScribe.Behaviour.TypedContract.new/1","ref":"ElixirScribe.Behaviour.TypedContract.html#c:new/1"},{"type":"callback","doc":"Accepts a map with the required and optional fields to create an Elixir Scribe Typed Contract.\n\nReturns the typed contract on successful creation, otherwise raises an error.","title":"ElixirScribe.Behaviour.TypedContract.new!/1","ref":"ElixirScribe.Behaviour.TypedContract.html#c:new!/1"},{"type":"callback","doc":"Returns the type specification for the Elixir Scribe Typed Contract.\n\nUsed internally by all this behaviour callbacks, but may also be useful in the case more advanced usage of Norm is required outside this struct.","title":"ElixirScribe.Behaviour.TypedContract.type_spec/0","ref":"ElixirScribe.Behaviour.TypedContract.html#c:type_spec/0"},{"type":"callback","doc":"Updates the Elixir Scribe Typed Contract for the given field and value.\n\nReturns `{:ok, struct}` on successful update, otherwise returns `{:error, reason}`.","title":"ElixirScribe.Behaviour.TypedContract.update/3","ref":"ElixirScribe.Behaviour.TypedContract.html#c:update/3"},{"type":"callback","doc":"Updates the Elixir Scribe Typed Contract for the given field and value.\n\nReturns the typed contract on successful update, otherwise raises an error.","title":"ElixirScribe.Behaviour.TypedContract.update!/3","ref":"ElixirScribe.Behaviour.TypedContract.html#c:update!/3"},{"type":"extras","doc":"# 🪶 Elixir Scribe 📜\n\n[![Github Sponsor: https://github.com/sponsors/Exadra37](/assets/svg/github-sponsor.svg)](https://github.com/sponsors/Exadra37)\n\nScribes were an elite in ancient societies, highly regarded and holding special social status. They were disciplined and known for their excellent craftsmanship in writing laws, copying documents, and other related tasks.\n\nThe motivation to create the Elixir Scribe tool was to encourage developers to write **Clean Code** in a **Clean Software Architecture**, to enable them to know in seconds all domains, resources, and actions used in a project, while reducing complexity and contributing for less technical debt.\n\nThe Elixir Scribe tool enables developers to go from zero to hero in no time, while empowering their craftsmanship and professionalism to reach new levels never imagined before or thought to not be easy to achieve.\n\nDon't just be a Developer or Engineer, become a Scribe Developer and Engineer 🚀\n\nCurious about the next release? Be no more: [check here](#elixir-scribe-next-release) for details and dates.\n\n> [!IMPORTANT]  \n>\n> → The Elixir Scribe tool is not about implementing Domain Driven Design (DDD), but may share some similarities.  \n\n\n> [!TIP]  \n>\n> → Clone the branch `WIP` to be on the bleeding edge of the current work being done.  \n> → Be prepared for buggy code. It may work or not.  \n> → Be ready for breaking changes, which can happen at any moment.  \n\n\n> [!CAUTION]  \n>  \n> → NEVER base your work on the `WIP` branch. Code and features may disappear.  \n> → Push forces can occur at any commit, thus commit history will be broken.","title":"🪶 Elixir Scribe 📜","ref":"readme.html"},{"type":"extras","doc":"* [Reduced Complexity Equals to Less Technical Debt](#reduced-complexity-equals-to-less-technical-debt)\n* [Clean Software Architecture](#clean-software-architecture)\n* [Clean Code](#clean-code)\n* [Benefits](#benefits)\n* [Installation](#installation)\n* [Quickstart](#quickstart)\n* [Documentation](#documentation)\n* [Contributing](#contributing)\n* [Roadmap](#roadmap)\n  + [Next Release](#elixir-scribe-next-release)\n  + [Generators](#elixir-scribe-generators)\n  + [Marketplace](#elixir-scribe-marketplace)","title":"TOC - 🪶 Elixir Scribe 📜","ref":"readme.html#toc"},{"type":"extras","doc":"The Elixir Scribe generators will help developers to effortless organize their code by Domain, Resource and all possible actions on the Resource to reduce complexity and technical debt.\n\nThis reduction in complexity translates into fewer bugs and makes it easier to add new features and implement bug fixes when they arise. Consequently, this results in less maintenance and technical debt across the lifetime of a project, leading to a more robust and easier-to-work-with code-base.\n\nUltimately, this enhances developer experience, velocity and productivity. This is a win win for both the business and developers.\n\n[TOC](#toc)","title":"Reduced Complexity Equals to Less Technical Debt - 🪶 Elixir Scribe 📜","ref":"readme.html#reduced-complexity-equals-to-less-technical-debt"},{"type":"extras","doc":"A Clean Software Architecture **MUST** allow developers to understand in seconds, or in a couple of minutes, all Domains, Resources and actions of the project they are working on.\n\nCan you grasp, in seconds, all domains, resources, and actions in your current large professional or side projects? With the Elixir Scribe folder structure you will always reply **YES** to the this question.\n\nLet's do an exercise. How much time do you need to know all domains, resources and actions from the below fictitious `acme` app?\n\nThe below folder structure reflects the Elixir Scribe generator used in the [Quickstart](#quickstart):\n\n```text\n$ tree -d -L 5 lib\n\nlib\n├── my_app\n│   └── domains\n│       ├── catalog\n│       │   ├── category\n│       │   │   ├── create\n│       │   │   ├── delete\n│       │   │   ├── edit\n│       │   │   ├── list\n│       │   │   ├── new\n│       │   │   ├── read\n│       │   │   └── update\n│       │   └── product\n│       │       ├── create\n│       │       ├── delete\n│       │       ├── edit\n│       │       ├── export\n│       │       ├── import\n│       │       ├── list\n│       │       ├── new\n│       │       ├── read\n│       │       └── update\n│       └── warehouse\n│           └── stock\n│               ├── export\n│               └── import\n└── my_app_web\n    ├── components\n    │   └── layouts\n    └── domains\n        ├── catalog\n        │   ├── category\n        │   │   ├── create\n        │   │   ├── delete\n        │   │   ├── edit\n        │   │   ├── list\n        │   │   ├── new\n        │   │   ├── read\n        │   │   └── update\n        │   └── product\n        │       ├── create\n        │       ├── delete\n        │       ├── edit\n        │       ├── export\n        │       ├── import\n        │       ├── list\n        │       ├── new\n        │       ├── read\n        │       └── update\n        └── warehouse\n            └── stock\n                ├── export\n                └── import\n\n```\n\n> **Domains:** catalog, warehouse  \n> **Resources:** category, product, stock  \n> **Actions:** create, delete, edit, export, import, list, new, read, update  \n\nSo, how many seconds did it took you to have an overview of the project and understand all it's Domains, Resources and Actions? How much time do you think you would spend looking through the code-base to find where to fix a bug to `export` a Product or to add a new feature to the Category resource?\n\nThis is a very simplistic view of a project. Now, imagine reaping the benefits of this folder structure implemented on your huge code-base, which may now contain dozens, hundreds, or even thousands of resources *(yes, I worked in such a project)*, each with potentially more actions than the ones exemplified here.\n\nTake a moment to compare this folder structure with the traditional ones used in any project you worked so far, be it an Elixir / Phoenix project or not, which often condenses several resources and all their actions into a single module / class, without even care about Domain boundaries. \n\n[TOC](#toc)","title":"Clean Software Architecture - 🪶 Elixir Scribe 📜","ref":"readme.html#clean-software-architecture"},{"type":"extras","doc":"Writing Clean Code relies on several aspects, and one of them is to follow the Single Responsibility Principle, which is encouraged by the Elixir Scribe tool, when it forces the developer to split all possible actions on a Resource into a single module by action on a Resource.\n\nThe Developer can still manage to mix responsibilities in the action module for the Resource. For example, any access to a third party service should be delegated to another module. The action module should do only one thing, to handle the data as per the businesses rules.\n\nThe Elixir Scribe tool doesn't enforce an architecture inside the action folder, leaving the Developer free to apply the best one for it's use case. For example, the module generated by the Elixir Scribe tool inside the action folder may be used as an entry-point to orchestrate all required steps to perform the action on the Resource, which will be done in other modules to follow the Single Responsibility Principle, reduce complexity per module, resulting in a Clean Code that it's easier to understand and maintain.\n\nCan you now understand why Elixir Scribe encourages developers to write **Clean Code** in a **Clean Software Architecture**?\n\n[TOC](#toc)","title":"Clean Code - 🪶 Elixir Scribe 📜","ref":"readme.html#clean-code"},{"type":"extras","doc":"The main benefits of using the Elixir Scribe Tool to create and maintain Elixir and Phoenix projects:\n\n* Know in seconds all domains, resources, and actions used in your project by just looking at the folder structure. This is priceless for anyone working on the project.\n* Newcomers to the project or anyone returning after a while can easily understand what the project is all about and quickly start working on new features or bugs (folder structure for the win).\n* The single responsibility principle is encouraged by the design of the folder structure, where a clear separation of domains, resources, and actions occurs, encouraging one resource action per module.\n* Helps to reduce tech debt due to the separation of concerns encouraged by the folder structure, which naturally guides developers to not mix new features into an existing module, whether it's a new domain, resource, or just an action on a resource.\n* It's much easier to find where to add a new feature without mixing it with existing code. For example: If it's a new action on a resource, just add one more action folder and the respective module(s) inside it.\n* Quickly find and start debugging a bug because actions aren't entangled in a huge module with several resources for a domain (context). For example, if the bug is about creating a resource on a given domain, it's straightforward to know where to start due to the folder structure.\n* Each resource on a domain has a public API to avoid direct access to the underlying implementation of each resource. This removes cross-domain boundary calls into the internals of each resource implementation, preventing the coupling of domains, which is a significant source of technical debt and complexity in a codebase. The Resource public API MUST be used not only from other domains but also from within the domain itself. This will allow the internals of a resource in a domain to change as needed, provided that we do not affect the public API for it.\n* The folder structure is documentation that never lies about what domains (contexts), resources, and actions are available in your project, whether huge or small.\n* Enables easier migration from a monolithic to a micro-services architecture when the need arises.\n\n[TOC](#toc)","title":"Benefits - 🪶 Elixir Scribe 📜","ref":"readme.html#benefits"},{"type":"extras","doc":"The [Elixir Scribe](https://hexdocs.pm/elixir_scribe/api-reference.html) package can be installed\nby adding `elixir_scribe` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:elixir_scribe, \">= 0.1.0\", only: :dev, runtime: false}\n  ]\nend\n```\n\n[TOC](#toc)","title":"Installation - 🪶 Elixir Scribe 📜","ref":"readme.html#installation"},{"type":"extras","doc":"> **IMPORTANT:** Not merged yet to the main branch.\n\nLet's create a fictitious Online Shop to exemplify how to use the Elixir Scribe tool:\n\n```\nmix phx.new my_app --database sqlite3\n```\n\nNow let's use the Elixir Scribe generators to create the domain `Catalog` and add to it the resource `Category` with the default actions:\n\n```\nmix scribe.gen.html Catalog Category categories name:string desc:string\n```\n\n> Elixir Scribe default actions: create, delete, edit, list, new, read, update\n\n\nLet's add to the domain `Catalog` the resource `Product` with custom actions on top of the default actions:\n\n```\nmix scribe.gen.html Catalog Product products name:string desc:string --actions import,export\n```\n\nLet's add the domain `Warehouse` with the resource `Stock` without default actions. We will need to provide custom actions:\n\n```\n mix scribe.gen.html Warehouse Stock stocks product_id:integer quantity:integer --actions import,export --no-default-actions\n```\n\n[TOC](#toc)","title":"Quickstart - 🪶 Elixir Scribe 📜","ref":"readme.html#quickstart"},{"type":"extras","doc":"The docs can be found at  .\n\nDocumentation is generated with [ExDoc](https://github.com/elixir-lang/ex_doc).\n\n[TOC](#toc)","title":"Documentation - 🪶 Elixir Scribe 📜","ref":"readme.html#documentation"},{"type":"extras","doc":"The Elixir Scribe tool is highly opinionated, therefore I ask you to first [open a discussion](https://github.com/Exadra37/elixir-scribe/discussions/new?category=ideas) to propose your idea to avoid working and then seeing the PR refused.\n\n[TOC](#toc)","title":"Contributing - 🪶 Elixir Scribe 📜","ref":"readme.html#contributing"},{"type":"extras","doc":"The roadmap is extensive and due to the continuous high efforts and commitments to develop and maintain everything I may only provide some of the features to Sponsors and/or as Pro versions that will require a paid license.","title":"Roadmap - 🪶 Elixir Scribe 📜","ref":"readme.html#roadmap"},{"type":"extras","doc":"The next release will be `0.3.0`, and will contain the following new generators:\n\n* `scribe.gen.domain`\n* `scribe.gen.html`\n\nThe release data is expected to be around the middle of July 2024.","title":"Elixir Scribe Next Release - 🪶 Elixir Scribe 📜","ref":"readme.html#elixir-scribe-next-release"},{"type":"extras","doc":"This generators provide developers with the tools to go from zero to hero in no time, while following a Clean Software Architecture that encourages Clean Code when building their Apps or APIs.\n\n#### Typed Contracts\n\n- [x] - Typed Contract is a struct with type specs to guarantee data shape correctness at runtime.\n\n\n#### Mix Tasks\n\n- [ ] `scribe.gen.domain`\n- [ ] `scribe.gen.html`\n- [ ] `scribe.gen.live`\n- [ ] `scribe.gen.template`\n  * Generates a template to be customized for the project.\n    + The template includes the code generated by all Elixir Scribe generators, e.g. `scribe.gen.*`.\n  * The customized template can also be uploaded to the marketplace as freebie or one to be sold under a paid license.\n- [ ] `scribe.gen.home`\n  * Removes current default Home page.\n  * Adds new Home page with links to each Domain and Resource.\n  * Optimizes default HTMl layout and components for a more clean and usable UI:\n    + Table headers in Bold\n    + Highlight links in blue, not in bold (black).\n- [ ] `scribe.gen.ci` \n  * Generates a CI file for Github or Gitlab with at least the following:\n    + `mix format --dry-run --check-formatted`\n    + `mix deps.unlock --check-unused`\n    + `mix deps.audit`\n    + `mix hex.audit`\n    + `mix sobelow`\n    + `mix credo`\n    + `mix doctor`\n    + `mix test --cover`\n- [ ] `scribe.gen.project`\n  * Generates all Domains, Resources and Actions from an Elixir Scribe spec module `%ElixirScribe.ProjectSpecs{}`. \n- [ ] `scribe.gen.api`\n  * Adds a dynamic API key (only valid for one request).\n  * Adds support for fingerprinting the mobile and web app.\n  * Adds support to collect metrics.\n  * Adds a Secure Remote Configuration for mobile apps to avoid a new release:\n    + To rotate certificate pins. This enables dynamic certificate pinning on the mobile app.\n    + To change any settings pre-configured within the release binary (API endpoints, Menus, etc.).\n    + To control feature flags.\n    + To enable a kill-switch to force upgrade to a new version.\n    + To dynamically conduct experiments with user experience, A/B testing.\n- [ ] `scribe.gen.auth`\n  * Adds enhanced User Authentication:\n    + fingerprinting of the client enabled by default \n    + enabled by default to all pages, except Home page.\n- [ ] `scribe.gen.oauth`\n  * Adds enhanced OAuth for popular providers:\n    + fingerprinting of the client enabled by default \n    + enabled by default to all pages, except Home page.\n- [ ] `scribe.gen.native`\n  * Enables the project to be built for Desktop, Android and Apple.\n  * Elixir libraries to build for native targets:\n    + [LiveView Native](https://github.com/liveview-native)\n    + [Elixir Desktop](https://github.com/elixir-desktop/desktop)\n    + [ExTauri](https://github.com/Exadra37/ex_tauri)\n- [ ] `scribe.gen.static`\n  * Generates a static website from a current project.\n- [ ] `scribe.gen.deploy`\n  * Adds easy deployment to popular hosting providers.\n- [ ] `scribe.gen.admin`","title":"Elixir Scribe Generators - 🪶 Elixir Scribe 📜","ref":"readme.html#elixir-scribe-generators"},{"type":"extras","doc":"- [ ] Website to sell free and paid licenses for Elixir Scribe templates.\n  * The marketplace will support as vendors anyone that builds quality templates with the Elixir Scribe tool.\n  * Vendors of paid licenses will pay a fee for each sale, plus payment fees. It will be all very transparent, no hidden fees.","title":"Elixir Scribe Marketplace - 🪶 Elixir Scribe 📜","ref":"readme.html#elixir-scribe-marketplace"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,48]}}